@@ -1,6 +1,7 @@
-from __future__ import print_function
 from __future__ import absolute_import
-from retriever import MODULE_LIST
+from __future__ import print_function
+
+from retriever.lib.scripts import MODULE_LIST
 
 
 def compile():
@@ -8,5 +9,6 @@ def compile():
     MODULE_LIST(force_compile=True)
     print("done.")
 
+
 if __name__ == "__main__":
     compile()
