@@ -3,16 +3,14 @@
 This module contains optional functions for importing data from Excel.
 
 """
-from builtins import str
 from builtins import object
+from builtins import str
 
 
 class Excel(object):
-
     @staticmethod
     def empty_cell(cell):
-        """Tests whether an excel cell is empty or contains only
-        whitespace"""
+        """Test if excel cell is empty or contains only whitespace."""
         if cell.ctype == 0:
             return True
         if str(cell.value).strip() == "":
@@ -21,7 +19,7 @@ class Excel(object):
 
     @staticmethod
     def cell_value(cell):
-        """Returns the string value of an excel spreadsheet cell"""
+        """Return string value of an excel spreadsheet cell."""
         if (cell.value).__class__.__name__ == 'unicode':
             return (str(cell.value).encode()).strip()
         return str(cell.value).strip()
